import numpy as np
import numpy.polynomial.polynomial as nppol# pour le polynôme
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker # pour le format des étiquettes en x (remplacer le point décimal par la virgule)

# Paramètres acido-basiques
Ke=1e-14# produit ionique de l'eau
Ca = 0.12
Cb = 0.10
Va = 10.0
Vbmax=20
#------------------------------------------------------------
AffichageConcentration=True
#réglage perso de la grille secondaire du graphe :
fig = plt.figure()
ax1 = fig.add_subplot(1, 1, 1)
minor_xticks = np.arange(0, 101, 1)#espaces de la grille
minor_yticks = np.arange(0, 101, 0.2)#espaces de la grille
ax1.set_xticks(minor_xticks, minor=True)
ax1.set_yticks(minor_yticks, minor=True)
ax1.grid(which='minor', alpha=0.2)
couleur2='darkorange'#axe vertical des concentrations

def concentration(Vb):#Calcule [H3O+]
    a=1# a, b et c les coefficients du polynôme
    b=(Ca*Va-Cb*Vb)/(Vb+Va)-Ke
    c=Ke
    vm = nppol.polyroots([-c,-b,a,1])#chercher les racines du polynôme
    positive=filter(lambda x: x>0,filter(np.isreal,vm))
    C=next(positive)
    return C

def titrage(Vmin,Vmax):# calcul du pH
    volume = np.linspace(Vmin, Vmax, 256,endpoint=True)
    ph=-np.log10(np.vectorize(concentration)(volume))
    Oxo=(Ca*Va-Cb*volume)/Va
    return volume,ph,Oxo

#---------------------------remplacer  les points décimaux par des virgules :
def func(x, pos):  # formatter function takes tick label and tick position
    s = str(x)
    ind = s.index('.')
    return s[:ind] + ',' + s[ind+1:]   # change dot to comma
x_format = ticker.FuncFormatter(func)  # make formatter
#----------------------------------------------------------------------------------------------------------

def graphique_ph(Vmin,Vmax):
    volume,ph, oxonium=titrage(Vmin,Vmax)
    ax1.plot(volume, ph)
    #Les lignes suivantes pour tracer [H3O+] :
    if AffichageConcentration==True:
        ax2=ax1.twinx()
        ax2.plot(volume, oxonium, ls='--', c=couleur2)
        plt.ylabel(r"[$H_3O^+$]",  c=couleur2, rotation=0, loc='top')
        ax2.tick_params(axis='y',  labelcolor=couleur2)
        ax2.set_ylim(0, Ca)

plt.title("Titrage pH-métrique d'un acide fort")
plt.xlabel(r'Volume $V_b$ de base versé (mL)')
plt.ylabel("pH", rotation=0, loc='top')

plt.grid()
ax1.axis([0, Vbmax, 0, 14])#limites des axes
plt.yticks(np.arange(0, 15, 1))# ajuster l'espace entre valeurs axe y
plt.xticks(np.arange(0, Vbmax, 2.0))# ajuster l'espace entre valeurs axe x
#    ax1.xaxis.set_major_formatter(x_format)  # appliquer le format de virgule décimale

graphique_ph(0,Vbmax)

fig.savefig('CourbeTitrageAcideFortV2.png', transparent=False, dpi=150)# Création image PNG (adapter la résolution au résultat souhaité)

plt.show()
